//+------------------------------------------------------------------+
//|                                                ay-mplib.v103.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, ahmad.yani@hotmail.com"
#property link      "http://eyesfx.web44.net"
#property library

#include <ay-mplib.const.v103.mqh> 

void MpGet
   (
   
        double    dparam[4]
      , int       iparam[4]
      , double    arates[][6]
      , double&   mpdatarow[][MPSIZE]  //--result: price per row (price, tpo, vol)     
      , int&      mplinesidx[MPLNSIZE]    //--result: lines index
      , double&   mptotal[MPTSIZE]        //--result: total (tpo, vol)
      , int&      mpletter[][MPLTSIZE]    //--result: letter per row 
   )
{

   //-->extract params
   double     hh           = dparam[MPDP_HH]
            , ll           = dparam[MPDP_LL]
            , onetick      = dparam[MPDP_ONETICK]
            , vapercent    = dparam[MPDP_VAPERCENT]
            ;
     
   int        startbar     = iparam[MPIP_STARTBAR]
            , endbar       = iparam[MPIP_ENDBAR] 
            , digits       = iparam[MPIP_DIGITS]  
            , step         = iparam[MPIP_STEP]            
            ;
   
   double ticksize = step*onetick;        
   //<--extract params   
    
   double   sp;                                       //--start price      
   double   midrangeprice = hh -( (hh-ll)/2.0 );     
   double   price, priceh, pricel; 
   double   barhp;                                    //--bar high price
   double   barlp;                                    //--bar low price
   double   barrange, rangeinrow;                     //--bar range
   double   pipvol;                                   //--volume per pip in bar range
   double   totaltpo, volinrow, totalvol, maxtpo, maxvol, maxpir;
   int      i, j; 
   int      maxtpoidx, maxvolidx, maxpiridx;            
   int      mprowcount  = MathRound((hh - ll)/ticksize);
   bool     inrow;
         

   ArrayResize(mpdatarow, mprowcount);
   ArrayResize(mpletter, mprowcount);    

   ArrayInitialize(mplinesidx, MPNOIDX);   
   ArrayInitialize(mptotal, 0.0);     
   ArrayInitialize(mpdatarow, 0.0);   
   ArrayInitialize(mpletter, MPNOIDX); 
        
   //-->price level per ticksize ( row )
   sp = hh;
   for (i=0; i<mprowcount; i++)
   {
      mpdatarow[i][MP_PRICE] = NormalizeDouble(sp, digits);
      sp -= ticksize;
   }
   //<--price level per ticksize ( row )
            
   //-->profile letter per row
   for (i=0; i<mprowcount; i++)
   {
      priceh = mpdatarow[i][MP_PRICE];
      pricel = priceh - ticksize;
    
      int letteridx = 0;
      int barnum    = 0;     
                     
      for (j=startbar; j>=endbar; j--)
      {
         
         barhp    = arates[j][_HIGH];
         barlp    = arates[j][_LOW];     
         barrange = ( (arates[j][_HIGH] - arates[j][_LOW]) / onetick); 
         barrange = MathMax(barrange, 0.000001);
         pipvol   = MathMax( arates[j][_VOL] / barrange, 0.000001 );   
                
         //-->check at least one price of bar prices (from high to low) is in row
         inrow = false;    
         price = priceh; 
         while (price > pricel)
         {

            if (    (price <= barhp || price - barhp < onetick)
                 && (price >  barlp)
               ) 
            { inrow = true; break; }            
            
            price -= onetick;
         }
         //-->check at least one price of bar prices (from high to low) is in row
         
         //-->tpo, vol, letters
         rangeinrow = 0.0;
         if ( inrow )
         {
                                                                                                          
            mpdatarow[i][MP_TPO] += 1.0;
            totaltpo                += 1.0; 
            
            rangeinrow  = ( MathMin(barhp, priceh) - MathMax(barlp, pricel) ) / onetick ;                      
            volinrow    = rangeinrow * pipvol;
            
            mpdatarow[i][MP_PIR] += rangeinrow;                                        
            mpdatarow[i][MP_VOL] += volinrow;                        
            totalvol                += volinrow;

            if (letteridx < MPLTSIZE) mpletter[i][letteridx] = barnum;
            
            letteridx++;
         }
         
         barnum++;
         //<--tpo, vol, letters

         //-->save maxtpo
         if ( mpdatarow[i][MP_TPO] > maxtpo) 
         {
            maxtpo    = mpdatarow[i][MP_TPO];
            maxtpoidx = i;               
         }
            
         if (mpdatarow[i][MP_TPO] == maxtpo) 
         {
            // take the closest to the middle of profile range
            if (  MathAbs(midrangeprice - mpdatarow[i][MP_PRICE]) 
                  < 
                  MathAbs(midrangeprice - mpdatarow[maxtpoidx][MP_PRICE]) 
               )
            {
               maxtpo    = mpdatarow[i][MP_TPO];
               maxtpoidx = i;
            }
         }
         //<--save maxtpo
            
         //-->save maxvol
         if (mpdatarow[i][MP_VOL] > maxvol) 
         {
            maxvol    = mpdatarow[i][MP_VOL];
            maxvolidx = i;               
         }
            
         if (mpdatarow[i][MP_VOL] == maxvol) 
         {
            // choose the nearest to the middle of profile range
            if (   MathAbs(midrangeprice - mpdatarow[i][MP_PRICE]) 
                   < 
                   MathAbs(midrangeprice - mpdatarow[maxvolidx][MP_PRICE]) 
               )
            {
               maxvol    = mpdatarow[i][MP_VOL];
               maxvolidx = i;
            }
         }
         //<--save maxvol   
         
         //-->save maxpir      
         if (mpdatarow[i][MP_PIR] > maxpir) 
         {
            maxpir    = mpdatarow[i][MP_PIR];
            maxpiridx = i;               
         }
            
         if (mpdatarow[i][MP_PIR] == maxpir) 
         {
            // choose the nearest to the middle of profile range
            if (   MathAbs(midrangeprice - mpdatarow[i][MP_PRICE]) 
                   < 
                   MathAbs(midrangeprice - mpdatarow[maxpiridx][MP_PRICE]) 
               )
            {
               maxpir    = mpdatarow[i][MP_PIR];
               maxpiridx = i;
            }
         }         
         //<--save maxpir
      }
   }
   //<--profile letter per row   

   mplinesidx[MPL_TPOPOC] = maxtpoidx;
   mplinesidx[MPL_VOLPOC] = maxvolidx;
   mplinesidx[MPL_MAXPIR] = maxpiridx;
   
   mptotal[MPT_TPO] = totaltpo;
   mptotal[MPT_VOL] = totalvol;   
     
   
   //-->calculate value area
   bool usevol = false;
   int vahidx, validx;
   for (i=0; i<2; i++)
   {                     
      vahidx=0; validx=0;
      if (i==1) usevol = true;
      
      _MPGetValueArea
      (        
           usevol
         , vapercent
         , mprowcount
         , mpdatarow
         , maxtpo
         , maxtpoidx      
         , totaltpo
         , maxvol
         , maxvolidx
         , totalvol
         , vahidx
         , validx
      );
      
      if (usevol)
      {
         mplinesidx[MPL_VOLVAH] = vahidx;
         mplinesidx[MPL_VOLVAL] = validx;                  
      }
      else
      {
         mplinesidx[MPL_TPOVAH] = vahidx;
         mplinesidx[MPL_TPOVAL] = validx;   
      }
   }
   //<--calculate value area
              

}

/**
 * MPGetValueArea                                                   |
 * Private func                                             
 */
void _MPGetValueArea
(
     bool         usevolprofile
   , double       vapercent
   , int          mpdatasize
   , double       mpdata[][3]
   , double       maxtpo
   , int          maxtpoidx      
   , double       totaltpo
   , double       maxvol
   , int          maxvolidx
   , double       totalvol
   , int          &vahidx
   , int          &validx
)
{

   int    idx     = maxtpoidx;            
   if(usevolprofile) idx = maxvolidx;   
   
   double total   = totaltpo;             
   if(usevolprofile) total = totalvol;
   
   double vatvpo  = (vapercent/100.0) * total;  
       
   double tvpo    = mpdata[idx][MP_TPO];  if (usevolprofile) tvpo = mpdata[idx][MP_VOL];   

   int     upperidx_dp     = idx //dual prices(2 steps), upper poc 
         , loweridx_dp     = idx //dual prices(2 steps), lower poc   
         , lastupperidx    = idx
         , lastloweridx    = idx;    
         
   bool   ontop=false, onbottom=false;
      
   double utvpo1, utvpo2, ltvpo1, ltvpo2
         , tvpo2upper
         , tvpo2lower;
   
   int     remainingtop    = lastupperidx    //--dont forget this include row 0
         , remainingbottom = (mpdatasize-1) - lastloweridx;
   /*
   if(!usevolprofile)
      Print("idx = ", idx);
      Print(
          "upperidx_dp: " , upperidx_dp, " tvpo2upper ", tvpo2upper, " lastupperidx: ", lastupperidx, " remainingtop ", remainingtop      
        , " ||| "
        , "loweridx_dp: " , loweridx_dp, " tvpo2lower ", tvpo2lower, " lastloweridx: ", lastloweridx, " remainingbottom ", remainingbottom      
       );   
   */
   
   while (tvpo <= vatvpo)   
   {      

      //-->new dual prices idx & checking for idx ontop/bottom      
      //-->upperidx
      if (remainingtop >= 2)
      { 
         upperidx_dp = lastupperidx - 2; 
         
         utvpo1 = mpdata[upperidx_dp+1][MP_TPO];  
         if (usevolprofile) utvpo1 = mpdata[upperidx_dp+1][MP_VOL];   
             
         utvpo2 = mpdata[upperidx_dp  ][MP_TPO];  
         if (usevolprofile) utvpo2 = mpdata[upperidx_dp  ][MP_VOL];                  
      }
      else 
      {
         ontop = true;
         
         if (remainingtop==1)
         {
            utvpo1 = mpdata[lastupperidx-1][MP_TPO];  
            if (usevolprofile) utvpo1 = mpdata[lastupperidx-1][MP_VOL];    
            
            utvpo2 = 0;   
         }
         else
         {
            utvpo1=0; utvpo2=0;
         }
      }
      //<--upperidx
      
      //-->loweridx
      if (remainingbottom >= 2)
      {
         loweridx_dp = lastloweridx + 2;    
         
         ltvpo1 = mpdata[loweridx_dp-1][MP_TPO]; 
         if (usevolprofile) ltvpo1 = mpdata[loweridx_dp-1][MP_VOL]; 
         
         ltvpo2 = mpdata[loweridx_dp  ][MP_TPO]; 
         if (usevolprofile) ltvpo2 = mpdata[loweridx_dp][MP_VOL];          
      }
      else
      {
         onbottom = true;
         //remainingbottom = (mpdatasize-1) - lastloweridx;
         if (remainingbottom==1)
         {
            ltvpo1 = mpdata[lastloweridx+1][MP_TPO]; 
            if (usevolprofile) ltvpo1 = mpdata[lastloweridx+1][MP_VOL];   
            
            ltvpo2 = 0;    
         }
         else
         {
            ltvpo1=0; ltvpo2=0;
         }         
      }
      //<--loweridx      
      //<--new dual prices idx

      //-->check if vatvo reached by single step       
      if ( (tvpo + utvpo1) >= vatvpo || (tvpo + ltvpo1) >= vatvpo )
      {
         
         if ( utvpo1 >= ltvpo1 )
         {
            lastupperidx--;   
            tvpo += utvpo1;      
            /*
            if(!usevolprofile)
            Print(
                "upperidx_dp: " , upperidx_dp, " tvpo2upper ", tvpo2upper, " lastupperidx: ", lastupperidx, " remainingtop ", remainingtop      
              , " ||| "
              , "loweridx_dp: " , loweridx_dp, " tvpo2lower ", tvpo2lower, " lastloweridx: ", lastloweridx, " remainingbottom ", remainingbottom      
              , " vatvpo ", vatvpo, " tvpo ", tvpo
             );      
            */        
            break;
         }
         else      
         { 
            lastloweridx++;
            tvpo += ltvpo1;
            /*
            if(!usevolprofile)
            Print(
                "upperidx_dp: " , upperidx_dp, " tvpo2upper ", tvpo2upper, " lastupperidx: ", lastupperidx, " remainingtop ", remainingtop      
              , " ||| "
              , "loweridx_dp: " , loweridx_dp, " tvpo2lower ", tvpo2lower, " lastloweridx: ", lastloweridx, " remainingbottom ", remainingbottom      
              , " vatvpo ", vatvpo, " tvpo ", tvpo
             );      
            */
            break;
         }                     
         
      }      
      //<--check if vatvo reached by single step      

      //--2 step prices
      tvpo2upper = utvpo1 + utvpo2;
      tvpo2lower = ltvpo1 + ltvpo2;
      
      //--keep using upper area when remaining row at bottom < 2
      if (!ontop && onbottom)
      {
         lastupperidx = upperidx_dp;
         tvpo += tvpo2upper;      
      }
      else
      //--keep using lower area when remaining row at top < 2
      if (ontop && !onbottom)
      {
         lastloweridx = loweridx_dp;
         tvpo += tvpo2lower;      
      }
      //--both upper and lower area had remaining row >=2
      else
      {            
         if (  tvpo2upper > tvpo2lower )
         {
            lastupperidx = upperidx_dp;
            tvpo += tvpo2upper;
         }
         else
         if (  tvpo2upper < tvpo2lower )
         {
            lastloweridx = loweridx_dp;
            tvpo += tvpo2lower;
         }
         else 
         if ( tvpo2upper == tvpo2lower )
         {
            //--choose the nearest to poc
            if (idx - upperidx_dp >= loweridx_dp - idx)
            {
               lastloweridx = loweridx_dp;
               tvpo += tvpo2lower;   
            }
            else
            {     
               lastupperidx = upperidx_dp;
               tvpo += tvpo2upper;
            }      
         }      
      }
      
      remainingtop    = lastupperidx; //dont forget this include row 0
      remainingbottom = (mpdatasize-1) - lastloweridx;
      /*
      if(!usevolprofile)
      Print(
          "upperidx_dp: " , upperidx_dp, " tvpo2upper ", tvpo2upper, " lastupperidx: ", lastupperidx, " remainingtop ", remainingtop      
        , " ||| "
        , "loweridx_dp: " , loweridx_dp, " tvpo2lower ", tvpo2lower, " lastloweridx: ", lastloweridx, " remainingbottom ", remainingbottom      
        , " vatvpo ", vatvpo, " tvpo ", tvpo
       );      
      */

      
   }
   //--end while (tvpo <= vatvpo)
   
   vahidx = lastupperidx;
   validx = lastloweridx;  
   /*
   if(!usevolprofile)   
   Print("lastupperidx ", lastupperidx, " lastloweridx ", lastloweridx);
   */
}